import { v4 } from 'uuid';
import * as frames from '@r/frames';

const JAIL_ID = `gtm-${v4()}`;
let loaded = false;
let PENDING_EVENTS = [];

const getJail = () => {
  const jail = document.getElementById(JAIL_ID);
  return jail && jail.contentWindow ? jail : null;
};

const queueEvent = (eventName, payload) => {
  PENDING_EVENTS.push({ eventName, payload });
};

export const getLocation = () => ({
  href: location.href,
  hostname: location.hostname,
  origin: location.origin,
  pathname: location.pathname || '/',
  search: location.search.replace(/^\?/, ''),
  hash: location.hash.replace(/^#/, ''),
});

export const trigger = (eventName, payload) => {
  const jail = getJail();

  // Make sure GTM is loaded, it might not be ready since
  // we load it asyncronously.
  if (!jail || !loaded) {
    queueEvent(eventName, payload);
    return;
  }

  if (payload) {
    frames.postMessage(jail.contentWindow, 'data.gtm', payload);
  }

  frames.postMessage(jail.contentWindow, 'event.gtm', {
    event: eventName,
  });
};

const onload = () => {
  const jail = getJail();

  if (!jail || !loaded) {
    // Not loaded yet.
    return;
  }

  // Explicitly copy the pending events to a new array and clear the queue.
  // `trigger` appends to PENDING_EVENTS so this is to prevent any issues
  // such as the queue growing forever (although that shouldn't happen)
  const pendingEvents = PENDING_EVENTS.slice();
  PENDING_EVENTS = [];

  pendingEvents.forEach(({ eventName, payload }) => trigger(eventName, payload));
};

export const setup = ({ containerId, jailUrl, payload }) => {
  if (!containerId) {
    // eslint-disable-next-line no-console
    console.warn('GTM#setup: No `containerId` specified, skipping.');
    return;
  }

  if (!jailUrl) {
    // eslint-disable-next-line no-console
    console.warn('GTM#setup: No `jailUrl` specified, skipping.');
    return;
  }

  // Ensure events are queued until the jail is completely loaded.
  frames.listen('gtm');
  frames.receiveMessageOnce('loaded.gtm', () => {
    loaded = true;
    frames.stopListening('gtm');
  });

  const jail = document.createElement('iframe');
  jail.style.display = 'none';
  jail.id = JAIL_ID;
  jail.name = JSON.stringify({
    referrer: document.referrer,
    ...getLocation(),
    ...payload,
  });

  jail.src = `${jailUrl}?id=${containerId}`;
  jail.onload = onload;

  document.body.appendChild(jail);
};
